/*-------------------------------------------------------------------
[SCRIPT] Service Broker Sample - Source database configuration   
[DATABASE] 
[DESCRIPTION] 
[MAJ PAR] DATAFLY - Arian Papillon
[DATEMAJ] SQL Saturday - 05/09/2015  
-------------------------------------------------------------------*/
-- Base Source
USE master
GO
CREATE DATABASE SBSource
GO
USE SBSource
GO
SELECT  [BusinessEntityID]
      , [PersonType]
      , [FirstName]
      , [LastName]
INTO    person
FROM    AdventureWorks2014.Person.Person 
GO
ALTER TABLE person
ADD CONSTRAINT pk_person PRIMARY KEY CLUSTERED ( BusinessEntityID )

----- Configuration du Service Broker --------------
USE master
ALTER DATABASE SBSource SET ENABLE_BROKER;

USE SBSource
-- Message Type
CREATE MESSAGE TYPE [DataModification_MessageType]
VALIDATION = WELL_FORMED_XML ;
GO

-- Contrat 
CREATE CONTRACT [DataModification_Contract]
(
[DataModification_MessageType]
SENT BY INITIATOR
)
GO

-- File d'attente
CREATE QUEUE DataModification_Queue 
WITH STATUS = ON

-- Service
CREATE SERVICE [DataModification_Service] 
ON QUEUE DataModification_Queue (DataModification_Contract)
GRANT SEND ON SERVICE::DataModification_Service TO PUBLIC

-------- Rseau -------------------------------
IF EXISTS (SELECT * FROM sys.endpoints WHERE name = 'BrokerEndpoint') DROP ENDPOINT BrokerEndpoint
CREATE ENDPOINT BrokerEndpoint STATE = STARTED AS TCP ( LISTENER_PORT = 4037 )
    FOR SERVICE_BROKER ( AUTHENTICATION = WINDOWS, ENCRYPTION = DISABLED );
GO

-- Route
CREATE ROUTE DataModification_Route  -- route to destination
WITH
SERVICE_NAME = 'DataSynchro_Service',
ADDRESS = 'TCP://192.168.56.1:4038';

SELECT * FROM sys.routes

------------------------------------------------------------------

-- Envoi message
BEGIN TRAN
DECLARE @IDConversation UNIQUEIDENTIFIER
BEGIN DIALOG CONVERSATION @IDConversation
FROM SERVICE [DataModification_Service] TO SERVICE
    'DataSynchro_Service'
ON CONTRACT [DataModification_Contract]
WITH ENCRYPTION = OFF
-- message de test
DECLARE @Msg xml;
SET @Msg = (SELECT 'dbo' AS '@TableSchema'
                  ,'person' AS '@TableName'
                  ,'update' AS '@Action'
                  ,'BusinessEntityID' AS '@KeyCol'
                  , (
                     SELECT column_name
                           ,data_type
                           ,CASE is_nullable
                            WHEN 'YES' THEN 'true' ELSE 'false'
                            END AS is_nullable
                           ,CHARACTER_MAXIMUM_LENGTH AS Charlen
                       FROM information_schema.columns [column]
                       WHERE table_name = 'person'
                       FOR XML AUTO,ROOT ('TableStructure') ,TYPE) 
                  , (
                     SELECT TOP (3) *
                       FROM Person Row
                       FOR XML AUTO,BINARY BASE64,ROOT ('TableData') ,TYPE)
                    FOR XML PATH ('DataModification')) ; 
SELECT @Msg;
SEND ON CONVERSATION @IDConversation MESSAGE TYPE [DataModification_MessageType] (@Msg);
COMMIT TRAN

-- o est le message ?
SELECT  * FROM    DataModification_Queue
SELECT * FROM sys.transmission_queue; 
SELECT  * FROM    sys.conversation_endpoints

--- Lorsque le destinataire a termin la conversation, traitons le message de fin de dialogue
    DECLARE @conversation UNIQUEIDENTIFIER
    DECLARE @msg xml
    DECLARE @msgType NVARCHAR(256)
    ;RECEIVE TOP(1) 
		@conversation = conversation_handle,
	    @msgType = message_type_name, 
		@msg = message_body
    FROM DataModification_Queue;
    IF (@msgType = 'http://schemas.microsoft.com/SQL/ServiceBroker/Error') OR
    	(@msgType = 'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog')
		BEGIN
    		END CONVERSATION @conversation
		END
 GO