/*-------------------------------------------------------------------
[SCRIPT] Service Broker Sample - Destination database configuration   
[DATABASE] 
[DESCRIPTION] 
[MAJ PAR] DATAFLY - Arian Papillon
[DATEMAJ] SQL Saturday - 05/09/2015  
-------------------------------------------------------------------*/

-- base destination
CREATE DATABASE SBDest
GO
USE SBDest
GO
SELECT  [BusinessEntityID]
      , [PersonType]
      , [FirstName]
      , [LastName]
  INTO    person
FROM    [DUKE\SQL2014].SBSource.dbo.Person 
go
ALTER TABLE person
ADD CONSTRAINT pk_person PRIMARY KEY CLUSTERED ( BusinessEntityID )

----- Configuration Service Broker --------------
USE master
ALTER DATABASE SBDest SET ENABLE_BROKER;

USE SBDest
GO
-- Message Type
CREATE MESSAGE TYPE [DataModification_MessageType]
VALIDATION = WELL_FORMED_XML ;
GO

-- Contrat 
CREATE CONTRACT [DataModification_Contract]
(
	[DataModification_MessageType]
	SENT BY INITIATOR
)
GO

-- File d'attente
CREATE QUEUE DataSynchro_Queue 
WITH STATUS = ON

-- Service
CREATE SERVICE [DataSynchro_Service] 
ON QUEUE DataSynchro_Queue (DataModification_Contract)
grant send on service::DataSynchro_Service to PUBLIC

-- Rseau
IF EXISTS (SELECT * FROM sys.endpoints WHERE name = 'BrokerEndpoint') DROP ENDPOINT BrokerEndpoint
CREATE ENDPOINT BrokerEndpoint
    STATE = STARTED
    AS TCP ( LISTENER_PORT = 4038 )
    FOR SERVICE_BROKER ( AUTHENTICATION = WINDOWS, ENCRYPTION = DISABLED ) ;
GO

-- Route
CREATE ROUTE DataModification_Route   -- route to source
    WITH
    SERVICE_NAME = 'DataModification_Service',
    ADDRESS = 'TCP://192.168.56.1:4037' ;

SELECT * FROM sys.routes
----------------------------------------------------------


-- Avons nous reu le message ?
-- O est mon message ?
SELECT * FROM DataSynchro_Queue 
SELECT * FROM sys.transmission_queue; 
SELECT * FROM sys.conversation_endpoints


-- voyons le message (sans le retirer de la queue)
DECLARE @msg VARBINARY(MAX); 
DECLARE @hdl UNIQUEIDENTIFIER; 
-- lecture du message 
SELECT TOP(1) @msg = message_body, @hdl = conversation_handle 
FROM   DataSynchro_Queue; 
-- utilisation du message 
SELECT CAST (@msg AS XML) AS 'Message reu' 


-- consommons le message 
DECLARE @hdl UNIQUEIDENTIFIER; 
DECLARE @msg XML;
DECLARE @msgType NVARCHAR(256);
RECEIVE TOP(1) @hdl = conversation_handle,
@msgType = message_type_name,
@msg = message_body
FROM    DataSynchro_Queue;
END CONVERSATION @HDL; -- Et fin de la conversation
SELECT @msg

SELECT * FROM DataSynchro_Queue -- le message a t consomm

-- Dcodons le message
------------------ Dcodage------------------------------
-- Action
DECLARE @XML_NATURE nvarchar (30) ,
        @TABLE_SCHEMA sysname,
        @TABLE_NAME sysname,
        @SQL_ACTION nvarchar (30) ,
        @KEY_COLUMN sysname,
        @ColumnList nvarchar (max) = '',
        @SQLCmd nvarchar (max) ;

SELECT @XML_NATURE = Txml1.XdataHead.value ('fn:local-name(.)','NVARCHAR(128)') 
      ,@TABLE_SCHEMA = Txml1.XdataHead.value ('@TableSchema','NVARCHAR(128)') 
      ,@TABLE_NAME = Txml1.XdataHead.value ('@TableName','NVARCHAR(128)') 
      ,@SQL_ACTION = Txml1.XdataHead.value ('@Action','NVARCHAR(128)') 
      ,@KEY_COLUMN = Txml1.XdataHead.value ('@KeyCol','NVARCHAR(128)')
  FROM @Msg.nodes ('./*') AS Txml1 (XdataHead) ;

-- structure de la table
IF OBJECT_ID ('tempdb..#XMLColumns') IS NOT NULL
    BEGIN
        DROP TABLE #XMLColumns;
    END;
SELECT x.value ('@column_name','sysname') AS column_name
      ,x.value ('@data_type','sysname') AS data_type
      ,x.value ('@is_nullable','VARCHAR(20)') AS is_nullable
      ,x.value ('@Charlen','VARCHAR(20)') AS Charlen INTO #XMLColumns
  FROM @Msg.nodes ('/DataModification/TableStructure/column') TempXML (x) ;

-- extraction des lignes
DECLARE @SQLExtract nvarchar (max) = 'SELECT ';
SELECT @SQLExtract = @SQLExtract + '
x.value(''@' + column_name + ''', ''' + data_type + CASE
                                                    WHEN Charlen IS NULL THEN '' ELSE '(' + Charlen + ')'
                                                    END + '''' + ') AS [' + column_name + '],'
  FROM #XMLColumns;
SET @SQLExtract = LEFT (@SQLExtract,LEN (@SQLExtract) - 1) ;
SELECT @SQLExtract = @SQLExtract + ' FROM @Msg.nodes(''/DataModification/TableData/Row'') TempXML (x)';

-- Gnration des commandes INSERT, UPDATE, DELETE
IF @SQL_ACTION = 'insert'
    BEGIN
        SET @SQLCmd = 'INSERT ' + @TABLE_SCHEMA + '.' + @TABLE_NAME + ' ' + @SQLExtract;
        PRINT @SQLCmd;
    END;
IF @SQL_ACTION = 'update'
    BEGIN
        SELECT @ColumnList = @ColumnList + column_name + '=E.' + column_name + ','
          FROM #XMLColumns
          WHERE column_name <> @KEY_COLUMN;
        SET @ColumnList = LEFT (@ColumnList,LEN (@ColumnList) - 1) ;
        SET @SQLCmd = 'UPDATE ' + @TABLE_SCHEMA + '.' + @TABLE_NAME + ' SET ' + @ColumnList + ' FROM ' + @TABLE_SCHEMA + '.' + @TABLE_NAME + ' AS T JOIN (' + @SQLExtract + ') AS E ON T.' + @KEY_COLUMN + '=E.' + @KEY_COLUMN;
        PRINT @SQLCmd;
    END;
IF @SQL_ACTION = 'delete'
    BEGIN
        SET @SQLCmd = 'DELETE ' + @TABLE_SCHEMA + '.' + @TABLE_NAME + ' FROM ' + @TABLE_SCHEMA + '.' + @TABLE_NAME + ' AS T JOIN (' + @SQLExtract + ') AS E ON T.' + @KEY_COLUMN + '=E.' + @KEY_COLUMN;
        PRINT @SQLCmd;
    END;
-- Excution
-- EXEC sp_executeSQL @SQLCmd,N'@Msg xml',@Msg = @Msg;





